/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia;

import java.util.List;

import javax.xml.namespace.QName;

import org.sourceprojects.lycia.LyciaBuilder.GenerationStrategy;
import org.w3c.dom.Element;
import org.xml.sax.ContentHandler;

/**
 * Base class to extend the {@link LyciaConfigurator} chain in Lycia. Can be
 * used to enhance the capabilities of legal tags.
 * 
 * @author noctarius
 * 
 * @param <B>
 *            Type of ContextObject to use
 */
public abstract class AbstractXmlParserConfigurator<B> implements
		LyciaConfigurator<B> {

	private final LyciaConfigurator<B> parent;
	private final LyciaParser<B> xmlParser;

	public AbstractXmlParserConfigurator(LyciaParser<B> xmlParser,
			LyciaConfigurator<B> parent) {

		if (parent == null)
			throw new NullPointerException(
					"For enhanced XmlParserConfigurators the parent configurator cannot be null");

		if (xmlParser == null)
			throw new NullPointerException("XmlParser instance cannot be null");

		this.xmlParser = xmlParser;
		this.parent = parent;
	}

	@Override
	public void addTagDefinitionParser(TagDefinitionParser<B> definitionParser) {
		getParentXmlParserConfigurator().addTagDefinitionParser(
				definitionParser);
	}

	@Override
	public void addTagDefinitionParser(Object definitionParser) {
		getParentXmlParserConfigurator().addTagDefinitionParser(
				definitionParser);
	}

	@Override
	public void addTagDefinitionParsers(
			TagDefinitionParser<B>[] definitionParsers) {

		getParentXmlParserConfigurator().addTagDefinitionParsers(
				definitionParsers);
	}

	@Override
	public void addTagDefinitionParsers(Object[] definitionParsers) {
		getParentXmlParserConfigurator().addTagDefinitionParsers(
				definitionParsers);
	}

	@Override
	public TagDefinitionParser<B> findTagDefinitionParser(QName localElement,
			Element element) {

		return getParentXmlParserConfigurator().findTagDefinitionParser(
				localElement, element);
	}

	@Override
	public TagDefinitionParser<B> findTagDefinitionParser(String tagName,
			String namespaceURI, Element element) {

		return getParentXmlParserConfigurator().findTagDefinitionParser(
				tagName, namespaceURI, element);
	}

	@Override
	public List<TagDefinitionParser<B>> findTagDefinitionParsers(
			QName localElement, Element element) {

		return getParentXmlParserConfigurator().findTagDefinitionParsers(
				localElement, element);
	}

	@Override
	public List<TagDefinitionParser<B>> findTagDefinitionParsers(
			String tagName, String namespaceURI, Element element) {

		return getParentXmlParserConfigurator().findTagDefinitionParsers(
				tagName, namespaceURI, element);
	}

	@Override
	public GenerationStrategy getGenerationStrategy() {
		return getParentXmlParserConfigurator().getGenerationStrategy();
	}

	@Override
	public GeneratorFactory<B> getGeneratorFactory() {
		return getParentXmlParserConfigurator().getGeneratorFactory();
	}

	@Override
	public LyciaConfigurator<B> getParentXmlParserConfigurator() {
		return parent;
	}

	@Override
	public void removeTagDefinitionParser(
			TagDefinitionParser<B> definitionParser) {

		getParentXmlParserConfigurator().removeTagDefinitionParser(
				definitionParser);
	}

	@Override
	public void removeTagDefinitionParser(Object definitionParser) {
		getParentXmlParserConfigurator().removeTagDefinitionParser(
				definitionParser);
	}

	@Override
	public void removeTagDefinitionParsers(
			TagDefinitionParser<B>[] definitionParsers) {

		getParentXmlParserConfigurator().removeTagDefinitionParsers(
				definitionParsers);
	}

	@Override
	public void removeTagDefinitionParsers(Object[] definitionParsers) {
		getParentXmlParserConfigurator().removeTagDefinitionParsers(
				definitionParsers);
	}

	@Override
	public void setGenerationStrategy(GenerationStrategy generationStrategy) {
		getParentXmlParserConfigurator().setGenerationStrategy(
				generationStrategy);
	}

	@Override
	public void setGeneratorFactory(GeneratorFactory<B> generatorFactory) {
		getParentXmlParserConfigurator().setGeneratorFactory(generatorFactory);
	}

	@Override
	public void addContentHandler(ContentHandler contentHandler) {
		parent.addContentHandler(contentHandler);
	}

	@Override
	public List<ContentHandler> getContentHandlers() {
		return parent.getContentHandlers();
	}

	@Override
	public void removeContentHandler(ContentHandler contentHandler) {
		parent.removeContentHandler(contentHandler);
	}

	protected LyciaParser<B> getXmlParser() {
		return xmlParser;
	}

}
