/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia;

import org.sourceprojects.lycia.LyciaBuilder.GenerationStrategy;

/**
 * Base interface for all GeneratorFactory implementations. Default
 * implementation is delivered by Lycia.
 * 
 * @author noctarius
 * @since 1.9.5
 * 
 * @param <B>
 *            Type of ContextObject to use
 */
public interface GeneratorFactory<B> {

	/**
	 * Creates a new instance of the {@link Generator} class configurated by
	 * this {@link GeneratorFactory}
	 * 
	 * @param generationStrategy
	 * @return instance of Generator
	 */
	Generator<B> newInstance(GenerationStrategy generationStrategy);

	/**
	 * Sets the class of the {@link Generator} implementation to use
	 * 
	 * @param generatorClass
	 *            Implementation to use as default {@link Generator}
	 */
	void setCustomGeneratorClass(Class<? extends Generator<B>> generatorClass);
}