/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.internal.generator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.xml.namespace.QName;

import org.sourceprojects.lycia.DefinitionBuilder;
import org.sourceprojects.lycia.Delegate;
import org.sourceprojects.lycia.ParserContext;
import org.sourceprojects.lycia.exceptions.LyciaParsingException;
import org.sourceprojects.lycia.internal.configuration.TagConfigurationUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * 
 * @author noctarius
 * @since 1.9.1
 */
class ReflectiveDelegate<B> implements Delegate<B> {
	private final Object parser;
	private final Method method;

	ReflectiveDelegate(Object parser, Method method) {
		this.parser = parser;
		this.method = method;
	}

	@Override
	public void invokeParse(Element element, String localName, QName parent,
			ParserContext<B> parserContext, DefinitionBuilder definitionBuilder)
			throws LyciaParsingException {
		try {
			Object[] parameters = TagConfigurationUtils.generateParameter(
					method, element, localName, parent, parserContext,
					definitionBuilder);
			method.invoke(parser, parameters);
		} catch (final IllegalAccessException e) {
			throw new LyciaParsingException("Parser method "
					+ method.getName() + " could not be executed", e);
		} catch (final InvocationTargetException e) {
			throw new LyciaParsingException("Parser method "
					+ method.getName() + " could not be executed", e);
		} catch (final IllegalArgumentException e) {
			throw new LyciaParsingException("Parser method "
					+ method.getName() + " could not be executed", e);
		}
	}

	@Override
	public Node invokeTransform(Element element, String localName,
			QName parent, ParserContext<B> parserContext,
			DefinitionBuilder definitionBuilder)
			throws LyciaParsingException {
		try {
			Object[] parameters = TagConfigurationUtils.generateParameter(
					method, element, localName, parent, parserContext,
					definitionBuilder);
			return (Node) method.invoke(parser, parameters);
		} catch (final IllegalAccessException e) {
			throw new LyciaParsingException("Transformer method "
					+ method.getName() + " could not be executed", e);
		} catch (final InvocationTargetException e) {
			throw new LyciaParsingException("Transformer method "
					+ method.getName() + " could not be executed", e);
		} catch (final IllegalArgumentException e) {
			throw new LyciaParsingException("Transformer method "
					+ method.getName() + " could not be executed", e);
		}
	}

	@Override
	public String toString() {
		return "ReflectiveDelegate [method=" + method + ", parser=" + parser
				+ "]";
	}

}
