/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.internal.validation;

import org.sourceprojects.lycia.ConstraintValidator;
import org.sourceprojects.lycia.annotations.DecimalMin;
import org.sourceprojects.lycia.exceptions.LyciaConfigurationException;

/**
 * @author noctarius
 * @since 1.9.1
 */
public class DecimalMinConstraintValidator implements
		ConstraintValidator<DecimalMin> {
	private DecimalMin annotation;

	@Override
	public void initialize(DecimalMin annotation) {
		this.annotation = annotation;
	}

	@Override
	public boolean isValid(Object object) {
		if (object == null) {
			return true;
		}

		try {
			double value = Double.parseDouble(object.toString());
			double expected = Double.parseDouble(annotation.value());

			return value > expected;
		} catch (NumberFormatException e) {
			throw new LyciaConfigurationException(
					"@DecimalMin could not be parsed", e);
		}
	}

}
