/*
 * 
 *  This file is part of Sourceprojects.org Lycia Copyright 2010 noctarius
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */
package org.sourceprojects.lycia.internal.validation;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;

import org.sourceprojects.lycia.ConstraintValidator;
import org.sourceprojects.lycia.annotations.Past;
import org.sourceprojects.lycia.exceptions.LyciaConfigurationException;

/**
 * @author noctarius
 * @since 1.9.1
 */
public class PastConstraintValidator implements ConstraintValidator<Past> {

	@Override
	public void initialize(Past annotation) {
	}

	@Override
	public boolean isValid(Object object) {
		if (object == null) {
			return true;
		}

		if (object instanceof Date) {
			return ((Date) object).after(new Date());
		}

		if (object instanceof Calendar) {
			return ((Calendar) object).after(new Date());
		}

		if (object instanceof String) {
			try {
				Date date = DateFormat.getInstance().parse((String) object);
				return date.after(new Date());
			} catch (ParseException e) {
				throw new LyciaConfigurationException(
						"Could not parse value for @Past annotation", e);
			}
		}

		throw new LyciaConfigurationException(
				"Could not handle value for @Past annotation");
	}

}
